/**
 * MainActivity.java
 * Jeff Ondich, 22 May 2014
 *
 * This is the main activity for the Threading sample project developed
 * for demonstration in Carleton's CS 342, Mobile Application Development.
 */

package com.jeffondich.threading.app;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;


public class MainActivity extends Activity implements View.OnClickListener {

    private DownloadHandler handler;
    private ProgressBar progressBar;
    private TextView resultsTextView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Button button = (Button) this.findViewById(R.id.slow_button);
        button.setOnClickListener(this);

        this.resultsTextView = (TextView)this.findViewById(R.id.slow_button_result);
        this.resultsTextView.setMovementMethod(new ScrollingMovementMethod());

        this.progressBar = (ProgressBar)this.findViewById(R.id.progress_bar);
        progressBar.setVisibility(View.GONE);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onClick(View view) {
        if (view.getId() == R.id.slow_button) {
            this.doSomethingSlow();
        }
    }

    /**
     * This sets up a download operation, intended to take enough time to illustrate
     * the use of a Handler to update a progress bar from a worker thread. The particular
     * server-side code (slowdownload.py) generates the specified number of lines of text,
     * and pauses for the specified number of milliseconds between the delivery of
     * successive lines.
     */
    private void doSomethingSlow() {
        String url = "http://cs.carleton.edu/faculty/jondich/courses/cs342_s14/programs/slowdownload.py";
        url += "?lines=2000";
        url += "&delay=5";
        File destination = new File(getFilesDir(), "slowfile.txt");
        int secondsAfterWhichToGiveUp = 10;

        HashMap<String, String> requestHeaders = new HashMap<String, String>();
        requestHeaders.put("User-Agent", "com.jeffondich.threading.app");

        if (this.handler == null) {
            this.handler = new DownloadHandler();
        }

        progressBar.setVisibility(View.VISIBLE);
        this.resultsTextView.setText("Loading...");

        Downloader downloader = new Downloader();
        downloader.download(url, destination, secondsAfterWhichToGiveUp, requestHeaders, this.handler);
    }

    /**
     * Our Handler subclass only needs to override handleMessage, which is used to respond
     * to any messages posted to our handler by the worker thread. See Downloader.java
     * for a brief discussion of the five message types Downloader may generate.
     */
    private class DownloadHandler extends Handler {
        @Override
        public void handleMessage(Message message) {
            TextView textView;

            switch (message.what) {
                case Downloader.HANDLE_SUCCESS:
                    progressBar.setVisibility(View.GONE);
                    textView = (TextView)findViewById(R.id.slow_button_result);
                    File downloadedFile = (File)message.obj;
                    if (textView != null && downloadedFile != null) {
                        String text = "";
                        try {
                            FileInputStream fileInputStream = new FileInputStream(downloadedFile);
                            BufferedReader reader = new BufferedReader(new InputStreamReader(fileInputStream));
                            String line;
                            while ((line = reader.readLine()) != null) {
                                text += line;
                                text += "\n";
                            }
                        } catch (IOException e) {
                            text += "The file arrived, but we can't read it, which is weird.";
                        }
                        textView.setText(text);
                    }
                    break;

                case Downloader.HANDLE_SET_PROGRESS_MAX:
                    progressBar.setMax(message.arg1);
                    progressBar.setProgress(0);
                    break;

                case Downloader.HANDLE_PROGRESS:
                    progressBar.setProgress(message.arg1);
                    Log.d("Progress [MainActivity]", String.format("(%d, %d)", progressBar.getProgress(), progressBar.getMax()));
                    break;

                case Downloader.HANDLE_TIMEOUT:
                    progressBar.setVisibility(View.GONE);
                    textView = (TextView)findViewById(R.id.slow_button_result);
                    if (textView != null) {
                        textView.setText("Timeout");
                    }
                    break;

                case Downloader.HANDLE_ERROR:
                    progressBar.setVisibility(View.GONE);
                    Exception e = (Exception)message.obj;
                    textView = (TextView)findViewById(R.id.slow_button_result);
                    if (textView != null && e != null) {
                        textView.setText(e.toString());
                    }
                    break;
            }
            super.handleMessage(message);
        }
    }
}
